@@ -30,13 +30,21 @@ class Server:
         self.rules = {}
 
         self.socket = socket.socket(family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0)
-        self.socket.bind((self.config.addr, self.config.port))
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
         self.socket.setblocking(False)
+
+        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
+        self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 1)
+        self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, 3)
+        self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 5)
+
+        self.socket.bind((self.config.addr, self.config.port))
         self.socket.listen(Server.MAX_NUM)
 
         self.outter_sockets = {}
 
+        self.network_name_to_inner_id = {}
+
         self.inner_conns = {}
         self.inner_ids = {}
 
@@ -50,10 +58,14 @@ class Server:
         for line in self.config.rules:
             self.add_rule(line)
 
-        logger.info("laurel start {}".format(json.dumps(self.config.__dict__)))
+        logger.info("[SERVER] start {}".format(json.dumps(self.config.__dict__)))
+
+    def choose_inner_conn(self, network_name: str = ''):
+        if network_name == '' or network_name not in self.network_name_to_inner_id:
+            cs = list(self.inner_conns.keys())
+        else:
+            cs = self.network_name_to_inner_id[network_name]
 
-    def choose_inner_conn(self):
-        cs = list(self.inner_conns.keys())
         if len(cs) == 0:
             return None
         return random.choice(cs)
@@ -62,9 +74,9 @@ class Server:
         if rule not in self.rules:
             try:
                 sk = socket.socket(family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0)
-                sk.bind((rule.to_addr, rule.to_port))
                 sk.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
                 sk.setblocking(False)
+                sk.bind((rule.to_addr, rule.to_port))
                 sk.listen(Server.MAX_NUM)
                 self.outter_sockets[sk] = rule
                 self.rsockets.append(sk)
@@ -92,9 +104,9 @@ class Server:
             del self.inner_ids[connection.conn]
             logger.info("[SERVER] close inner conn {}".format(inner_id))
 
-    def open_outter_conn(self, outter_id: str, conn: socket):
+    def open_outter_conn(self, network_name: str, outter_id: str, conn: socket):
         if outter_id not in self.outter_conns:
-            inner_id = self.choose_inner_conn()
+            inner_id = self.choose_inner_conn(network_name)
             if inner_id is None:
                 logger.error("[SERVER] no client for outter conn {}".format(outter_id))
                 return
@@ -134,8 +146,8 @@ class Server:
                 rule = self.outter_sockets[rsocket]
 
                 conn, addr = rsocket.accept()
-                outter_id = "{},{},{},{}".format(rule.from_addr, rule.from_port, addr[0], addr[1])
-                self.open_outter_conn(outter_id, conn)
+                outter_id = "{},{},{},{},{}".format(rule.network_name, rule.from_addr, rule.from_port, addr[0], addr[1])
+                self.open_outter_conn(rule.network_name, outter_id, conn)
 
                 if outter_id in self.outter_id_to_inner_id:
                     inner_id = self.outter_id_to_inner_id[outter_id]
@@ -162,6 +174,16 @@ class Server:
                             if ec != 0:
                                 break
 
+                            # register msg
+                            if msg.type == MsgType.REG:
+                                network_name = msg.id
+                                if network_name not in self.network_name_to_inner_id:
+                                    self.network_name_to_inner_id[network_name] = []
+                                self.network_name_to_inner_id[network_name].append(inner_id)
+
+                                logger.info("[SERVER] inner REG msg: network_name={}, inner_id={}".format(network_name, inner_id))
+                                continue
+
                             if ec == 0 and msg.id in self.outter_conns:
                                 outter_conn = self.outter_conns[msg.id]
                                 util.push(outter_conn.output_buf, msg.data)
